@@ -11,7 +11,6 @@ use mlua::prelude::*;
 use std::cell::RefCell;
 use std::io::{stdout, Write};
 use std::rc::Rc;
-//use std::collections::HashMap;
 
 pub const TROPICAL: &str = include_str!("../themes/tropical.lua");
 pub const GALAXY: &str = include_str!("../themes/galaxy.lua");
@@ -62,6 +61,7 @@ Ox has an ecosystem of plug-ins that you can make use of, they are as follows:
 ──────────────────── Code Helpers ────────────────────
 AutoIndent - A plug-in that will insert and remove code indentation automatically
 Pairs - A plug-in that will insert end brackets and end quotes automatically
+QuickComment - A plug-in that will help you quickly comment and uncomment lines of code
 
 ─────────────────── Web Development ──────────────────
 Emmet - A neat language to help you write HTML quickly - requires python and the py-emmet module
@@ -124,6 +124,7 @@ impl Theme {
 pub enum Plugin {
     AutoIndent,
     Pairs,
+    QuickComment,
     DiscordRPC,
     Emmet,
     Git,
@@ -144,6 +145,7 @@ impl Plugin {
         match self {
             Self::AutoIndent => "autoindent",
             Self::Pairs => "pairs",
+            Self::QuickComment => "quickcomment",
             Self::DiscordRPC => "discord_rpc",
             Self::Emmet => "emmet",
             Self::Git => "git",
@@ -192,7 +194,7 @@ impl Default for Assistant {
             scroll_sensitivity: 2,
             cursor_wrap: true,
             // Plug-ins
-            plugins: vec![Plugin::AutoIndent, Plugin::Pairs],
+            plugins: vec![Plugin::AutoIndent, Plugin::Pairs, Plugin::QuickComment],
             // Misc
             icons: false,
         }
@@ -490,6 +492,7 @@ impl Assistant {
                 &[
                     "autoindent",
                     "pairs",
+                    "quickcomment",
                     "emmet",
                     "live_html",
                     "discord_rpc",
@@ -505,6 +508,7 @@ impl Assistant {
             let plugin = match adding.as_str() {
                 "autoindent" => Plugin::AutoIndent,
                 "pairs" => Plugin::Pairs,
+                "quickcomment" => Plugin::QuickComment,
                 "emmet" => Plugin::Emmet,
                 "live_html" => Plugin::LiveHTML,
                 "discord_rpc" => Plugin::DiscordRPC,
@@ -765,7 +769,7 @@ impl Assistant {
         for plugin in &self.plugins {
             result += &plugin.to_config();
             if plugin == &Plugin::Git {
-                result += "git = { icons = true }";
+                result += "git = { icons = true }\n";
             }
         }
         // Ready to go
