@@ -10,19 +10,40 @@ use mlua::prelude::*;
 impl LuaUserData for Editor {
     fn add_fields<'lua, F: LuaUserDataFields<'lua, Self>>(fields: &mut F) {
         fields.add_field_method_get("cursor", |_, editor| {
-            let loc = editor.doc().char_loc();
-            Ok(LuaLoc {
-                x: loc.x,
-                y: loc.y + 1,
-            })
+            if let Some(doc) = editor.try_doc() {
+                let loc = doc.char_loc();
+                Ok(Some(LuaLoc {
+                    x: loc.x,
+                    y: loc.y + 1,
+                }))
+            } else {
+                Ok(None)
+            }
+        });
+        fields.add_field_method_get("selection", |_, editor| {
+            if let Some(doc) = editor.try_doc() {
+                let loc = doc.cursor.selection_end;
+                Ok(Some(LuaLoc {
+                    x: editor.doc().character_idx(&loc),
+                    y: loc.y + 1,
+                }))
+            } else {
+                Ok(None)
+            }
         });
         fields.add_field_method_get("document_name", |_, editor| {
-            let name = editor.doc().file_name.clone();
-            Ok(name)
+            if let Some(doc) = editor.try_doc() {
+                Ok(Some(doc.file_name.clone()))
+            } else {
+                Ok(None)
+            }
         });
         fields.add_field_method_get("document_length", |_, editor| {
-            let len = editor.doc().len_lines();
-            Ok(len)
+            if let Some(doc) = editor.try_doc() {
+                Ok(Some(doc.len_lines()))
+            } else {
+                Ok(None)
+            }
         });
         fields.add_field_method_get("version", |_, _| Ok(VERSION));
         fields.add_field_method_get("current_document_id", |_, editor| Ok(editor.ptr));
@@ -34,16 +55,31 @@ impl LuaUserData for Editor {
                 .map_or("Unknown".to_string(), |t| t.name))
         });
         fields.add_field_method_get("file_name", |_, editor| {
-            let name = get_file_name(&editor.doc().file_name.clone().unwrap_or_default());
-            Ok(name)
+            if let Some(doc) = editor.try_doc() {
+                Ok(Some(get_file_name(
+                    &doc.file_name.clone().unwrap_or_default(),
+                )))
+            } else {
+                Ok(None)
+            }
         });
         fields.add_field_method_get("file_extension", |_, editor| {
-            let name = get_file_ext(&editor.doc().file_name.clone().unwrap_or_default());
-            Ok(name)
+            if let Some(doc) = editor.try_doc() {
+                Ok(Some(get_file_ext(
+                    &doc.file_name.clone().unwrap_or_default(),
+                )))
+            } else {
+                Ok(None)
+            }
         });
         fields.add_field_method_get("file_path", |_, editor| {
-            let name = get_absolute_path(&editor.doc().file_name.clone().unwrap_or_default());
-            Ok(name)
+            if let Some(doc) = editor.try_doc() {
+                Ok(Some(get_absolute_path(
+                    &doc.file_name.clone().unwrap_or_default(),
+                )))
+            } else {
+                Ok(None)
+            }
         });
     }
 
@@ -206,6 +242,24 @@ impl LuaUserData for Editor {
             editor.update_highlighter();
             Ok(())
         });
+        methods.add_method_mut("cursor_snap", |_, editor, ()| {
+            editor.doc_mut().old_cursor = editor.doc().loc().x;
+            Ok(())
+        });
+        methods.add_method_mut("move_line_up", |_, editor, ()| {
+            let _ = editor.doc_mut().swap_line_up();
+            editor.hl_edit(editor.doc().loc().y);
+            editor.hl_edit(editor.doc().loc().y + 1);
+            editor.update_highlighter();
+            Ok(())
+        });
+        methods.add_method_mut("move_line_down", |_, editor, ()| {
+            let _ = editor.doc_mut().swap_line_down();
+            editor.hl_edit(editor.doc().loc().y.saturating_sub(1));
+            editor.hl_edit(editor.doc().loc().y);
+            editor.update_highlighter();
+            Ok(())
+        });
         // Cursor selection and clipboard
         methods.add_method_mut("select_up", |_, editor, ()| {
             editor.select_up();
@@ -232,6 +286,20 @@ impl LuaUserData for Editor {
             editor.update_highlighter();
             Ok(())
         });
+        methods.add_method_mut("select_to", |_, editor, (x, y): (usize, usize)| {
+            let y = y.saturating_sub(1);
+            editor.doc_mut().select_to(&Loc { y, x });
+            editor.update_highlighter();
+            Ok(())
+        });
+        methods.add_method_mut("cancel_selection", |_, editor, ()| {
+            editor.doc_mut().cancel_selection();
+            Ok(())
+        });
+        methods.add_method_mut("cursor_to_viewport", |_, editor, ()| {
+            editor.doc_mut().bring_cursor_in_viewport();
+            Ok(())
+        });
         methods.add_method_mut("cut", |_, editor, ()| {
             editor.plugin_active = true;
             if let Err(err) = editor.cut() {
@@ -449,11 +517,13 @@ impl LuaUserData for Editor {
         });
         methods.add_method_mut("move_next_match", |_, editor, query: String| {
             editor.next_match(&query);
+            editor.doc_mut().cancel_selection();
             editor.update_highlighter();
             Ok(())
         });
         methods.add_method_mut("move_previous_match", |_, editor, query: String| {
             editor.prev_match(&query);
+            editor.doc_mut().cancel_selection();
             editor.update_highlighter();
             Ok(())
         });
